# SPDX-License-Identifier: BSD-3-Clause
# Copyright (c) 2024 PyDims contributors (https://github.com/pydims)
from __future__ import annotations

from pydims.string_unit import StringUnit as _Unit

dimensionless = _Unit()

Unit = _Unit
